@@ -14,8 +14,7 @@
 
 class KSelectionOwner;
 
-class KManagerSelectionTest
-    : public QObject
+class KManagerSelectionTest : public QObject
 {
     Q_OBJECT
 public:
@@ -24,6 +23,7 @@ private Q_SLOTS:
     void testInitiallyOwned();
     void testLostOwnership();
     void testWatching();
+
 private:
     void claim(KSelectionOwner *owner, bool force = false, bool forceKill = true);
     void xSync();
@@ -34,20 +34,19 @@ class KSelectionWatcher;
 // For checking whether several signal have or have not been emitted,
 // QSignalSpy::wait() is not powerful enough for that (it is still
 // needed to do the event processing though). TODO: check if this is still true.
-class SigCheckOwner
-    : public QObject
+class SigCheckOwner : public QObject
 {
     Q_OBJECT
 public:
     SigCheckOwner(const KSelectionOwner &owner);
 private Q_SLOTS:
     void lostOwnership();
+
 public:
     bool lostownership;
 };
 
-class SigCheckWatcher
-    : public QObject
+class SigCheckWatcher : public QObject
 {
     Q_OBJECT
 public:
@@ -55,6 +54,7 @@ public:
 private Q_SLOTS:
     void newOwner();
     void lostOwner();
+
 public:
     bool newowner;
     bool lostowner;
